@@ -3,9 +3,9 @@ use 5.010;
 use Moose;
 use namespace::clean -except => 'meta';
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
-with qw(Hailo::Role::Generic
+with qw(Hailo::Role::Arguments
         Hailo::Role::Tokenizer);
 
 # output -> tokens
